from __future__ import annotations
import aiosqlite
from collections import namedtuple
from aiosqlite.core import Connection
from .errors import RecordNotFound

__all__ = ("LevelDB",)


class LevelDB:
    def __init__(self) -> None:
        self.CollectionTuple = namedtuple("Column", ["id", "xp", "level"])

    async def connection(self) -> Connection:
        return await aiosqlite.connect("level.db")

    async def sqlexec(self, query: str, entities=None) -> None:
        async with (con := await self.connection()).cursor() as cur:
            if not entities:
                await cur.execute(query)
            else:
                await cur.execute(query, entities)
            await con.commit()
            await con.close()

    @classmethod
    async def create(cls) -> "LevelDB":
        await (cl := cls()).sqlexec(
            "CREATE TABLE IF NOT EXISTS ranks(id integer PRIMARY KEY, xp INTEGER, level INTEGER)"
        )
        return cl

    async def read_data(self, user_id: int) -> tuple[int, ...] | None:
        async with (con := await self.connection()).cursor() as cur:
            if result := (
                await (
                    await cur.execute(
                        "SELECT id, xp, level FROM ranks WHERE id = ?", (user_id,)
                    )
                ).fetchone()
            ):
                return self.CollectionTuple(result[0], result[1], result[2])

    async def create_account(
        self, user_id: int, xp: int, level: int
    ) -> tuple[int, ...]:
        if not (res := await self.read_data(user_id)):
            await self.sqlexec(
                "INSERT INTO ranks(id, xp, level) VALUES(?, ?, ?)",
                (
                    user_id,
                    xp,
                    level,
                ),
            )
            return self.CollectionTuple(user_id, xp, level)
        return self.CollectionTuple(res[0], res[1], res[2])

    async def update_account(
        self, user_id: int, *, xp: int = None, level: int = None
    ) -> bool:
        if not (user := await self.read_data(user_id)):
            raise RecordNotFound
        if xp and not level:
            await self.sqlexec(
                "UPDATE ranks SET xp = ? WHERE id = ?",
                (
                    xp,
                    user_id,
                ),
            )
            return True
        elif level and not xp:
            await self.sqlexec(
                "UPDATE ranks SET level = ? WHERE id = ?",
                (
                    level,
                    user_id,
                ),
            )
            return True
        elif xp and level:
            await self.sqlexec(
                "UPDATE ranks SET xp = ? , level = ? WHERE id = ?",
                (
                    xp,
                    level,
                    user_id,
                ),
            )
            return True
        return False
