from __future__ import annotations
import aiosqlite
from collections import namedtuple
from aiosqlite.core import Connection
from .errors import RecordNotFound

__all__ = ("WarnDB",)

class WarnDB:
    def __init__(self) -> None:
        self.CollectionTuple = namedtuple("Column", ["id", "warns", "banned"])

    async def connection(self) -> Connection:
        return await aiosqlite.connect("warns.db")

    async def sqlexec(self, query: str, entities=None) -> None:
        async with (con := await self.connection()).cursor() as cur:
            if not entities:
                await cur.execute(query)
            else:
                await cur.execute(query, entities)
            await con.commit()
            await con.close()

    @classmethod
    async def create(cls) -> "WarnDB":
        await (cl := cls()).sqlexec(
            "CREATE TABLE IF NOT EXISTS warnings(id integer PRIMARY KEY, warns INTEGER, banned BOOLEAN)"
        )
        return cl

    async def read_data(self, user_id: int) -> tuple[int, ...]:
        async with (con := await self.connection()).cursor() as cur:
            if result := (
                await (
                    await cur.execute(
                        "SELECT id, warns, banned FROM warnings WHERE id = ?", (user_id,)
                    )
                ).fetchone()
            ):
                return self.CollectionTuple(result[0], result[1], result[2])

    async def create_account(
        self, user_id: int, warns: int, banned: bool
    ) -> tuple[int, ...]:
        res = await self.read_data(user_id)
        if not (res := await self.read_data(user_id)):
            await self.sqlexec(
                "INSERT INTO warnings(id, warns, banned) VALUES(?, ?, ?)",
                (
                    user_id,
                    warns,
                    banned,
                ),
            )
            return self.CollectionTuple(user_id, warns, banned)
        return self.CollectionTuple(res[0], res[1], res[2])

    async def update_account(
        self, user_id: int, *, warns: int = None, banned: bool = None
    ) -> bool:
        if not (user := await self.read_data(user_id)):
            raise RecordNotFound
        if warns and not banned:
            await self.sqlexec(
                "UPDATE warnings SET warns = ? WHERE id = ?",
                (
                    warns,
                    user_id,
                ),
            )
            return True
        elif banned and not warns:
            await self.sqlexec(
                "UPDATE warnings SET banned = ? WHERE id = ?",
                (
                    banned,
                    user_id,
                ),
            )
            return True
        elif warns and banned:
            await self.sqlexec(
                "UPDATE warnings SET warns = ? , banned = ? WHERE id = ?",
                (
                    warns,
                    banned,
                    user_id,
                ),
            )
            return True
        return False
